#!/bin/bash

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

set -e

#===================================================================================================
# Store the downloaded packages in one place
OPENVINO_INTEL_NPU_DRIVERS_DIR=/tmp/openvino_intel_npu_drivers
mkdir $OPENVINO_INTEL_NPU_DRIVERS_DIR

#===================================================================================================
# Set url base locations
LEVEL_ZERO_URL=https://github.com/oneapi-src/level-zero/releases/download
OPENVINO_INTEL_NPU_DRIVERS_URL=https://storage.openvinotoolkit.org/dependencies/npu/drivers

#===================================================================================================
# MTL NPU 2023.1 Driver for Ubuntu 22.04

os=$( . /etc/os-release ; echo "${ID}${VERSION_ID}" )
if [ "$os" != "ubuntu22.04" ]; then
    echo "MTL NPU Linux UMD PV1 driver is supported for Ubuntu 22.04 only" >&2
    exit 1
fi

OPENVINO_INTEL_NPU_DRIVERS_2023_1_DIR=$OPENVINO_INTEL_NPU_DRIVERS_DIR/2023.1
mkdir $OPENVINO_INTEL_NPU_DRIVERS_2023_1_DIR
OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR=$OPENVINO_INTEL_NPU_DRIVERS_2023_1_DIR/ubuntu22.04
mkdir $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR

OPENVINO_INTEL_NPU_DRIVERS_2023_1_URL=$OPENVINO_INTEL_NPU_DRIVERS_URL/2023.1
OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_URL=$OPENVINO_INTEL_NPU_DRIVERS_2023_1_URL/ubuntu22.04

LEVEL_ZERO_1_10_0_URL=$LEVEL_ZERO_URL/v1.10.0
LEVEL_ZERO_1_10_0_PACKAGE_NAME=level-zero_1.10.0+u22.04_amd64.deb

#===================================================================================================
# Find the latest package
wget --directory-prefix $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_URL/latest_release.txt
LATEST_DRIVER_VERSION=$(cat $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR/latest_release.txt)

#===================================================================================================
# Download all deb packages from LATEST_DRIVER_VERSION folder.
# Those packages are NPU Linux UMD driver, NPU compiler in driver and NPU firmware.
wget -r -l 1 --no-host-directories --cut-dirs=5 --accept *.deb --directory-prefix $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_URL/$LATEST_DRIVER_VERSION
wget --directory-prefix $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR $LEVEL_ZERO_1_10_0_URL/$LEVEL_ZERO_1_10_0_PACKAGE_NAME

#===================================================================================================
# Install all downloaded packages.
dpkg -i $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR/$LATEST_DRIVER_VERSION/*.deb
dpkg -i $OPENVINO_INTEL_NPU_DRIVERS_2023_1_UBUNTU_22_04_DIR/$LEVEL_ZERO_1_10_0_PACKAGE_NAME

#===================================================================================================
# Clean up. Remove the packages as they've already installed.
rm -rf $OPENVINO_INTEL_NPU_DRIVERS_DIR
